import * as i0 from '@angular/core';
import { Injectable, Optional, NgModule, SkipSelf } from '@angular/core';
import { of } from 'rxjs';
import moment from 'moment';
import * as i2 from '@angular/common/http';
import { CommonModule } from '@angular/common';

class ServiceConfig {
}

const defaultConfig = {
    apiUrl: 'http://utilityapi',
    usersEndpoint: '/sharepoint/users',
    listItemsEndpoint: '/sharepoint/listitems',
    username: 'viewer',
    password: 'viewer',
    photos: {
        site: 'https://sharepoint.cleavelandprice.com/digitaldisplay',
        list: 'Employee Thumbnail Photos',
        additionalPropertiesToRetrieve: [
            'EncodedAbsUrl'
        ]
    },
    caching: {
        enabled: true,
        lifeMinutes: 720
    },
    logging: {
        dataCached: true,
        cachedDataReturned: true,
        freshDataReturned: true,
        cacheCleared: true
    }
};

//#region Imports
//#endregion
class SharepointService {
    //#endregion
    //#region Initialization
    // If a ServiceConfig object is not provided when the module is imported, then default values are pulled in from default-config.ts.
    constructor(config, http) {
        this.config = config;
        this.http = http;
        this.cache = null;
        this.defaultCacheLifeMinutes = 720;
        this.config = this.config ?? defaultConfig;
        this.configureCaching();
        this.configureLogging();
    }
    //#region Utilities
    cacheExpired(cache) {
        if (!cache) {
            return true;
        }
        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
        const expiration = moment(cache.lastUpdate)
            .add(lifeMinutes, 'minutes');
        return moment().isAfter(expiration);
    }
    configureCaching() {
        // If caching config is missing entirely, give it a default
        this.config.caching = this.config.caching ?? {
            enabled: false,
            lifeMinutes: this.defaultCacheLifeMinutes
        };
        this.cache = {
            photos: null,
            users: null
        };
    }
    configureLogging() {
        // If logging config is missing entirely, give it a default
        this.config.logging = this.config.logging ?? {
            dataCached: false,
            cachedDataReturned: false,
            freshDataReturned: false,
            cacheCleared: false
        };
    }
    //#endregion
    getListItems(request) {
        const apiUrl = this.config?.apiUrl ?? defaultConfig.apiUrl;
        const endpoint = this.config?.listItemsEndpoint ?? defaultConfig.listItemsEndpoint;
        return this.http.post(`${apiUrl}${endpoint}`, request);
    }
    getPhotos() {
        const request = {
            username: this.config?.username ?? defaultConfig.username,
            password: this.config?.password ?? defaultConfig.password,
            site: this.config?.photos.site ?? defaultConfig.photos.site,
            list: this.config?.photos.list ?? defaultConfig.photos.list,
            additionalPropertiesToRetrieve: defaultConfig.photos.additionalPropertiesToRetrieve
        };
        const observable$ = this.getListItems(request);
        if (this.config.caching?.enabled) {
            if (this.cacheExpired(this.cache?.photos)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.photos = {
                        data,
                        lastUpdate: moment()
                    };
                    if (this.config.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Photos cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached photos');
                }
                return of(this.cache.photos.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config.logging?.freshDataReturned) {
            console.log('Returning fresh photos');
        }
        return observable$;
    }
    getUsers(request, id) {
        const apiUrl = this.config?.apiUrl ?? defaultConfig.apiUrl;
        const endpoint = this.config?.usersEndpoint ?? defaultConfig.usersEndpoint;
        const observable$ = this.http.post(`${apiUrl}${endpoint}/${id ?? ''}`, request);
        //#region Single user
        if (id) {
            return observable$;
        }
        //#endregion
        //#region Multiple users
        if (this.config.caching?.enabled) {
            if (this.cacheExpired(this.cache.users)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.users = {
                        data: data,
                        lastUpdate: moment()
                    };
                    if (this.config.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Users cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.cache.users.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config.logging?.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
        //#endregion
    }
    clearCache() {
        this.cache = null;
        if (this.config.logging?.cacheCleared) {
            console.log('Cache cleared');
        }
    }
}
SharepointService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointService, deps: [{ token: ServiceConfig, optional: true }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
SharepointService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiceConfig, decorators: [{
                    type: Optional
                }] }, { type: i2.HttpClient }]; } });

class SharepointModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('SharepointModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(config) {
        return {
            ngModule: SharepointModule,
            providers: [
                SharepointService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
}
SharepointModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointModule, deps: [{ token: SharepointModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
SharepointModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.0", ngImport: i0, type: SharepointModule, imports: [CommonModule] });
SharepointModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: SharepointModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ]
                }]
        }], ctorParameters: function () { return [{ type: SharepointModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { ServiceConfig, SharepointModule, SharepointService };
//# sourceMappingURL=cleavelandprice-ngx-lib-sharepoint.mjs.map
